/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_SLAMSERVICE_SLAMSERVICE_SKELETON_H
#define JAC_AP_SLAMSERVICE_SLAMSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/slamservice/slamservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace slamservice {
namespace skeleton {
namespace events
{
    using PositionEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::position>;
    using AttributeEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::attribute>;
    using LinePointEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::line_point>;
    using LineEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::line>;
    using SpeedLimitEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::speed_limit>;
    using TrafficObjectEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::traffic_object>;
    using LaneEventSlam = ara::com::internal::skeleton::event::EventAdapter<::dpc::Lane>;
    using RoadEdgeEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::RoadEdge>;
    using SLAMRoadModelEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::slam_road_model>;
    using AdjacentExtendLaneStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::AdjacentExtendLaneStruct>;
    using SMAStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::SMAStruct>;
    using WPExtendLaneStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::WPExtendLaneStruct>;
    using RemindWayPointStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::RemindWayPointStruct>;
    using ControlWayPointStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::ControlWayPointStruct>;
    using MsgHeaderEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::MsgHeader>;
    using TransitLaneStructEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::TransitLaneStruct>;
    using ControlWayPointsEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::ControlWaypoints>;
    using SLAMOdometryEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::slam_odometry>;
    static constexpr ara::com::internal::EntityId SLAMServicePositionEventId = 18038U; //PositionEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceAttributeEventId = 36926U; //AttributeEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceLinePointEventId = 45027U; //LinePointEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceLineEventId = 45904U; //LineEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceSpeedLimitEventId = 28595U; //SpeedLimitEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceTrafficObjectEventId = 6716U; //TrafficObjectEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceLaneEventSlamId = 55536U; //LaneEventSlam_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceRoadEdgeEventId = 62920U; //RoadEdgeEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceSLAMRoadModelEventId = 5227U; //SLAMRoadModelEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceAdjacentExtendLaneStructEventId = 49460U; //AdjacentExtendLaneStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceSMAStructEventId = 18332U; //SMAStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceWPExtendLaneStructEventId = 10669U; //WPExtendLaneStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceRemindWayPointStructEventId = 56635U; //RemindWayPointStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceControlWayPointStructEventId = 61573U; //ControlWayPointStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceMsgHeaderEventId = 27525U; //MsgHeaderEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceTransitLaneStructEventId = 7649U; //TransitLaneStructEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceControlWayPointsEventId = 33464U; //ControlWayPointsEvent_event_hash
    static constexpr ara::com::internal::EntityId SLAMServiceSLAMOdometryEventId = 55549U; //SLAMOdometryEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class SLAMServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultPositionEvent = PositionEvent.Initialize();
        ThrowError(resultPositionEvent);
        const ara::core::Result<void> resultAttributeEvent = AttributeEvent.Initialize();
        ThrowError(resultAttributeEvent);
        const ara::core::Result<void> resultLinePointEvent = LinePointEvent.Initialize();
        ThrowError(resultLinePointEvent);
        const ara::core::Result<void> resultLineEvent = LineEvent.Initialize();
        ThrowError(resultLineEvent);
        const ara::core::Result<void> resultSpeedLimitEvent = SpeedLimitEvent.Initialize();
        ThrowError(resultSpeedLimitEvent);
        const ara::core::Result<void> resultTrafficObjectEvent = TrafficObjectEvent.Initialize();
        ThrowError(resultTrafficObjectEvent);
        const ara::core::Result<void> resultLaneEventSlam = LaneEventSlam.Initialize();
        ThrowError(resultLaneEventSlam);
        const ara::core::Result<void> resultRoadEdgeEvent = RoadEdgeEvent.Initialize();
        ThrowError(resultRoadEdgeEvent);
        const ara::core::Result<void> resultSLAMRoadModelEvent = SLAMRoadModelEvent.Initialize();
        ThrowError(resultSLAMRoadModelEvent);
        const ara::core::Result<void> resultAdjacentExtendLaneStructEvent = AdjacentExtendLaneStructEvent.Initialize();
        ThrowError(resultAdjacentExtendLaneStructEvent);
        const ara::core::Result<void> resultSMAStructEvent = SMAStructEvent.Initialize();
        ThrowError(resultSMAStructEvent);
        const ara::core::Result<void> resultWPExtendLaneStructEvent = WPExtendLaneStructEvent.Initialize();
        ThrowError(resultWPExtendLaneStructEvent);
        const ara::core::Result<void> resultRemindWayPointStructEvent = RemindWayPointStructEvent.Initialize();
        ThrowError(resultRemindWayPointStructEvent);
        const ara::core::Result<void> resultControlWayPointStructEvent = ControlWayPointStructEvent.Initialize();
        ThrowError(resultControlWayPointStructEvent);
        const ara::core::Result<void> resultMsgHeaderEvent = MsgHeaderEvent.Initialize();
        ThrowError(resultMsgHeaderEvent);
        const ara::core::Result<void> resultTransitLaneStructEvent = TransitLaneStructEvent.Initialize();
        ThrowError(resultTransitLaneStructEvent);
        const ara::core::Result<void> resultControlWayPointsEvent = ControlWayPointsEvent.Initialize();
        ThrowError(resultControlWayPointsEvent);
        const ara::core::Result<void> resultSLAMOdometryEvent = SLAMOdometryEvent.Initialize();
        ThrowError(resultSLAMOdometryEvent);
    }

    SLAMServiceSkeleton& operator=(const SLAMServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId, mode)),
              PositionEvent(ptr->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              AttributeEvent(ptr->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              LinePointEvent(ptr->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              LineEvent(ptr->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              SpeedLimitEvent(ptr->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              TrafficObjectEvent(ptr->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              LaneEventSlam(ptr->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              RoadEdgeEvent(ptr->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              SLAMRoadModelEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              AdjacentExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              SMAStructEvent(ptr->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              WPExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              RemindWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              ControlWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              MsgHeaderEvent(ptr->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              TransitLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              ControlWayPointsEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
              SLAMOdometryEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec, mode)),
              PositionEvent(ptr->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              AttributeEvent(ptr->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              LinePointEvent(ptr->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              LineEvent(ptr->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              SpeedLimitEvent(ptr->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              TrafficObjectEvent(ptr->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              LaneEventSlam(ptr->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              RoadEdgeEvent(ptr->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              SLAMRoadModelEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              AdjacentExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              SMAStructEvent(ptr->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              WPExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              RemindWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              ControlWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              MsgHeaderEvent(ptr->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              TransitLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              ControlWayPointsEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
              SLAMOdometryEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer, mode)),
              PositionEvent(ptr->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              AttributeEvent(ptr->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              LinePointEvent(ptr->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              LineEvent(ptr->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              SpeedLimitEvent(ptr->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              TrafficObjectEvent(ptr->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              LaneEventSlam(ptr->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              RoadEdgeEvent(ptr->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              SLAMRoadModelEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              AdjacentExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              SMAStructEvent(ptr->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              WPExtendLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              RemindWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              ControlWayPointStructEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              MsgHeaderEvent(ptr->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              TransitLaneStructEvent(ptr->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              ControlWayPointsEvent(ptr->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
              SLAMOdometryEvent(ptr->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              PositionEvent(std::move(other.PositionEvent)),
              AttributeEvent(std::move(other.AttributeEvent)),
              LinePointEvent(std::move(other.LinePointEvent)),
              LineEvent(std::move(other.LineEvent)),
              SpeedLimitEvent(std::move(other.SpeedLimitEvent)),
              TrafficObjectEvent(std::move(other.TrafficObjectEvent)),
              LaneEventSlam(std::move(other.LaneEventSlam)),
              RoadEdgeEvent(std::move(other.RoadEdgeEvent)),
              SLAMRoadModelEvent(std::move(other.SLAMRoadModelEvent)),
              AdjacentExtendLaneStructEvent(std::move(other.AdjacentExtendLaneStructEvent)),
              SMAStructEvent(std::move(other.SMAStructEvent)),
              WPExtendLaneStructEvent(std::move(other.WPExtendLaneStructEvent)),
              RemindWayPointStructEvent(std::move(other.RemindWayPointStructEvent)),
              ControlWayPointStructEvent(std::move(other.ControlWayPointStructEvent)),
              MsgHeaderEvent(std::move(other.MsgHeaderEvent)),
              TransitLaneStructEvent(std::move(other.TransitLaneStructEvent)),
              ControlWayPointsEvent(std::move(other.ControlWayPointsEvent)),
              SLAMOdometryEvent(std::move(other.SLAMOdometryEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                PositionEvent = std::move(other.PositionEvent);
                AttributeEvent = std::move(other.AttributeEvent);
                LinePointEvent = std::move(other.LinePointEvent);
                LineEvent = std::move(other.LineEvent);
                SpeedLimitEvent = std::move(other.SpeedLimitEvent);
                TrafficObjectEvent = std::move(other.TrafficObjectEvent);
                LaneEventSlam = std::move(other.LaneEventSlam);
                RoadEdgeEvent = std::move(other.RoadEdgeEvent);
                SLAMRoadModelEvent = std::move(other.SLAMRoadModelEvent);
                AdjacentExtendLaneStructEvent = std::move(other.AdjacentExtendLaneStructEvent);
                SMAStructEvent = std::move(other.SMAStructEvent);
                WPExtendLaneStructEvent = std::move(other.WPExtendLaneStructEvent);
                RemindWayPointStructEvent = std::move(other.RemindWayPointStructEvent);
                ControlWayPointStructEvent = std::move(other.ControlWayPointStructEvent);
                MsgHeaderEvent = std::move(other.MsgHeaderEvent);
                TransitLaneStructEvent = std::move(other.TransitLaneStructEvent);
                ControlWayPointsEvent = std::move(other.ControlWayPointsEvent);
                SLAMOdometryEvent = std::move(other.SLAMOdometryEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::PositionEvent GetPositionEvent()
        {
            return std::move(PositionEvent);
        }
        events::AttributeEvent GetAttributeEvent()
        {
            return std::move(AttributeEvent);
        }
        events::LinePointEvent GetLinePointEvent()
        {
            return std::move(LinePointEvent);
        }
        events::LineEvent GetLineEvent()
        {
            return std::move(LineEvent);
        }
        events::SpeedLimitEvent GetSpeedLimitEvent()
        {
            return std::move(SpeedLimitEvent);
        }
        events::TrafficObjectEvent GetTrafficObjectEvent()
        {
            return std::move(TrafficObjectEvent);
        }
        events::LaneEventSlam GetLaneEventSlam()
        {
            return std::move(LaneEventSlam);
        }
        events::RoadEdgeEvent GetRoadEdgeEvent()
        {
            return std::move(RoadEdgeEvent);
        }
        events::SLAMRoadModelEvent GetSLAMRoadModelEvent()
        {
            return std::move(SLAMRoadModelEvent);
        }
        events::AdjacentExtendLaneStructEvent GetAdjacentExtendLaneStructEvent()
        {
            return std::move(AdjacentExtendLaneStructEvent);
        }
        events::SMAStructEvent GetSMAStructEvent()
        {
            return std::move(SMAStructEvent);
        }
        events::WPExtendLaneStructEvent GetWPExtendLaneStructEvent()
        {
            return std::move(WPExtendLaneStructEvent);
        }
        events::RemindWayPointStructEvent GetRemindWayPointStructEvent()
        {
            return std::move(RemindWayPointStructEvent);
        }
        events::ControlWayPointStructEvent GetControlWayPointStructEvent()
        {
            return std::move(ControlWayPointStructEvent);
        }
        events::MsgHeaderEvent GetMsgHeaderEvent()
        {
            return std::move(MsgHeaderEvent);
        }
        events::TransitLaneStructEvent GetTransitLaneStructEvent()
        {
            return std::move(TransitLaneStructEvent);
        }
        events::ControlWayPointsEvent GetControlWayPointsEvent()
        {
            return std::move(ControlWayPointsEvent);
        }
        events::SLAMOdometryEvent GetSLAMOdometryEvent()
        {
            return std::move(SLAMOdometryEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = PositionEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = AttributeEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LinePointEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LineEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = SpeedLimitEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = TrafficObjectEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = LaneEventSlam.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = RoadEdgeEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = SLAMRoadModelEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = AdjacentExtendLaneStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = SMAStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = WPExtendLaneStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = RemindWayPointStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = ControlWayPointStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = MsgHeaderEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = TransitLaneStructEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = ControlWayPointsEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = SLAMOdometryEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::PositionEvent PositionEvent;
        events::AttributeEvent AttributeEvent;
        events::LinePointEvent LinePointEvent;
        events::LineEvent LineEvent;
        events::SpeedLimitEvent SpeedLimitEvent;
        events::TrafficObjectEvent TrafficObjectEvent;
        events::LaneEventSlam LaneEventSlam;
        events::RoadEdgeEvent RoadEdgeEvent;
        events::SLAMRoadModelEvent SLAMRoadModelEvent;
        events::AdjacentExtendLaneStructEvent AdjacentExtendLaneStructEvent;
        events::SMAStructEvent SMAStructEvent;
        events::WPExtendLaneStructEvent WPExtendLaneStructEvent;
        events::RemindWayPointStructEvent RemindWayPointStructEvent;
        events::ControlWayPointStructEvent ControlWayPointStructEvent;
        events::MsgHeaderEvent MsgHeaderEvent;
        events::TransitLaneStructEvent TransitLaneStructEvent;
        events::ControlWayPointsEvent ControlWayPointsEvent;
        events::SLAMOdometryEvent SLAMOdometryEvent;
    };
    explicit SLAMServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId, mode)),
          PositionEvent(skeletonAdapter->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          AttributeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          LinePointEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          LineEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          SpeedLimitEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          TrafficObjectEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          LaneEventSlam(skeletonAdapter->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          RoadEdgeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          SLAMRoadModelEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          AdjacentExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          SMAStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          WPExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          RemindWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          ControlWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          MsgHeaderEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          TransitLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          ControlWayPointsEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId),
          SLAMOdometryEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit SLAMServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec, mode)),
          PositionEvent(skeletonAdapter->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          AttributeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          LinePointEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          LineEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          SpeedLimitEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          TrafficObjectEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          LaneEventSlam(skeletonAdapter->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          RoadEdgeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          SLAMRoadModelEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          AdjacentExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          SMAStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          WPExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          RemindWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          ControlWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          MsgHeaderEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          TransitLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          ControlWayPointsEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec),
          SLAMOdometryEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit SLAMServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer, mode)),
          PositionEvent(skeletonAdapter->GetSkeleton(), events::SLAMServicePositionEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          AttributeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAttributeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          LinePointEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLinePointEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          LineEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceLineEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          SpeedLimitEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSpeedLimitEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          TrafficObjectEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTrafficObjectEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          LaneEventSlam(skeletonAdapter->GetSkeleton(), events::SLAMServiceLaneEventSlamId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          RoadEdgeEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRoadEdgeEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          SLAMRoadModelEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMRoadModelEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          AdjacentExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceAdjacentExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          SMAStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSMAStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          WPExtendLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceWPExtendLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          RemindWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceRemindWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          ControlWayPointStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          MsgHeaderEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceMsgHeaderEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          TransitLaneStructEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceTransitLaneStructEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          ControlWayPointsEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceControlWayPointsEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer),
          SLAMOdometryEvent(skeletonAdapter->GetSkeleton(), events::SLAMServiceSLAMOdometryEventId, ::jac::ap::slamservice::SLAMService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    SLAMServiceSkeleton(const SLAMServiceSkeleton&) = delete;

    SLAMServiceSkeleton(SLAMServiceSkeleton&&) = default;
    SLAMServiceSkeleton& operator=(SLAMServiceSkeleton&&) = default;
    SLAMServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          PositionEvent(token.GetPositionEvent()),
          AttributeEvent(token.GetAttributeEvent()),
          LinePointEvent(token.GetLinePointEvent()),
          LineEvent(token.GetLineEvent()),
          SpeedLimitEvent(token.GetSpeedLimitEvent()),
          TrafficObjectEvent(token.GetTrafficObjectEvent()),
          LaneEventSlam(token.GetLaneEventSlam()),
          RoadEdgeEvent(token.GetRoadEdgeEvent()),
          SLAMRoadModelEvent(token.GetSLAMRoadModelEvent()),
          AdjacentExtendLaneStructEvent(token.GetAdjacentExtendLaneStructEvent()),
          SMAStructEvent(token.GetSMAStructEvent()),
          WPExtendLaneStructEvent(token.GetWPExtendLaneStructEvent()),
          RemindWayPointStructEvent(token.GetRemindWayPointStructEvent()),
          ControlWayPointStructEvent(token.GetControlWayPointStructEvent()),
          MsgHeaderEvent(token.GetMsgHeaderEvent()),
          TransitLaneStructEvent(token.GetTransitLaneStructEvent()),
          ControlWayPointsEvent(token.GetControlWayPointsEvent()),
          SLAMOdometryEvent(token.GetSLAMOdometryEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~SLAMServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&SLAMServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::PositionEvent PositionEvent;
    events::AttributeEvent AttributeEvent;
    events::LinePointEvent LinePointEvent;
    events::LineEvent LineEvent;
    events::SpeedLimitEvent SpeedLimitEvent;
    events::TrafficObjectEvent TrafficObjectEvent;
    events::LaneEventSlam LaneEventSlam;
    events::RoadEdgeEvent RoadEdgeEvent;
    events::SLAMRoadModelEvent SLAMRoadModelEvent;
    events::AdjacentExtendLaneStructEvent AdjacentExtendLaneStructEvent;
    events::SMAStructEvent SMAStructEvent;
    events::WPExtendLaneStructEvent WPExtendLaneStructEvent;
    events::RemindWayPointStructEvent RemindWayPointStructEvent;
    events::ControlWayPointStructEvent ControlWayPointStructEvent;
    events::MsgHeaderEvent MsgHeaderEvent;
    events::TransitLaneStructEvent TransitLaneStructEvent;
    events::ControlWayPointsEvent ControlWayPointsEvent;
    events::SLAMOdometryEvent SLAMOdometryEvent;
};
} // namespace skeleton
} // namespace slamservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_SLAMSERVICE_SLAMSERVICE_SKELETON_H
